﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.PackingEncoding.GroupingVector {
  [Item("Exhaustive Change Grouping Move Generator", "Generates all possible ChangeGrouping moves from a given groupingVector.")]
  [StorableClass]
  public class ExhaustiveChangeGroupingMoveGenerator : ChangeGroupingMoveGenerator, IExhaustiveMoveGenerator {
    [StorableConstructor]
    protected ExhaustiveChangeGroupingMoveGenerator(bool deserializing) : base(deserializing) { }
    protected ExhaustiveChangeGroupingMoveGenerator(ExhaustiveChangeGroupingMoveGenerator original, Cloner cloner) : base(original, cloner) { }
    public ExhaustiveChangeGroupingMoveGenerator() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ExhaustiveChangeGroupingMoveGenerator(this, cloner);
    }

    public static IEnumerable<ChangeGroupingMove> Generate(GroupingVectorEncoding groupingVector, int nrOfBins) {
      for (int i = 0; i < groupingVector.GroupingVector.Length; i++)
        for (int j = 0; j < nrOfBins; j++) {
          if (j != groupingVector.GroupingVector[i])
            yield return new ChangeGroupingMove(i, j, groupingVector);
        }
    }

    public static ChangeGroupingMove[] Apply(GroupingVectorEncoding groupingVector) {
      int nrOfBins = 0;
      foreach (int binNr in groupingVector.GroupingVector) {
        if (binNr > nrOfBins)
          nrOfBins = binNr;
      }
      nrOfBins++;

      int totalMoves = (groupingVector.GroupingVector.Length) * (nrOfBins - 1);
      ChangeGroupingMove[] moves = new ChangeGroupingMove[totalMoves];
      int count = 0;
      foreach (ChangeGroupingMove move in Generate(groupingVector, nrOfBins)) {
        moves[count++] = move;
      }
      return moves;
    }

    protected override ChangeGroupingMove[] GenerateMoves(GroupingVectorEncoding groupingVector) {
      return Apply(groupingVector);
    }
  }
}
